# Introduction

The Truffle implementation of the R FFI is based on the Truffle implementation of LLVM intermediate code, named [Sulong](https://github.com/graalvm/sulong).


# Building
Special setup is required to build FastR to use the Truffle R FFI implementation.

## Pre-Requisites

The DragonEgg plugin requires that `gcc 4.6` and `gfortran 4.6` be available.

On Mac OS, these can be installed with MacPorts (recommended) or Brew. Having installed these, set the following environment variables:

    export SULONG_GPP=/opt/local/bin/g++-mp-4.6
    export SULONG_GFORTRAN=/opt/local/bin/gfortran-mp-4.6
    export SULONG_GCC=/opt/local/bin/gcc-mp-4.6

The above definitions assume a MacPorts installation.

On Linux, the installation is system dependent, but once installed, set the same environment variables.

## Building Sulong
The `sulong` repository must be cloned to a sibling directory of `fastr` and built:

    cd $FASTR_HOME
    git clone https://github.com/graalvm/sulong.git
    cd sulong
    mx su-pulldragonegg

The `mx su-pulldragonegg` step is required to be able to compile Fortran code to LLVM, which is required by FastR.. As well as downloading DragonEgg this will also download and clang 3.2, which is needed to build DragonEgg. On Linux, it is necessary to use clang 3.2 in preference to any installed clang, e.g., 3.8, as some of the LLVM code generated by DragonEgg is syntax-incompatible with 3.8. The downloaded version is saved in `cache/tools/llvm/bin`. Since DragonEgg only needs to be built once per platform, the shared library can be copied from the sulong tree (search for `draghonegg.so`) into a shared location foir use with FastR.

Once `dragonegg` is built, unset the `SULONG_GCC`, `SULONG_GFORTRAN` and `SULONG_GPP` environment variables.

Sulong must be built with a more recent version of `clang` than 3.2, which means that `sulong` and `fastr` cannot be built in one step.
First make sure that you have a supported version of `clang` and related tools installed, e.g., 3.8 and that they are on your `PATH`. Also if you are on MacOS and are using MacPorts, you must make symbolic links to the explicitly versioned tool names, i.e. `clang` -> `clang-mp-3.8`. This also applies to the `opt` and `llvm-link` tools. You can set these links directly in `/opt/local/bin` using `sudo` or create a local `bin` directory and place the links there, making sure that this directory is on your `PATH`.

 and this (absolute) path should be added to `PATH`.

Now the remainder of sulong can be built.

    mx build

The `mx build` step will clone the `graal` repository, if necessary, and build the `truffle` suite also.

## Building FastR

To ensure that an LLVM build variant is chosen, set:

    export FASTR_RFFI=llvm

N.B. It is necessary to inform the build of the location of the various required libraries, e.g. `libpcre` as these are loaded explicitly from the `lib` directory at runtime. The build copies these libraries from wherever they were installed to the `lib` directory. For example on Ubuntu, set:

    export PKG_LDFLAGS_OVERRIDE=-L/lib/x86_64-linux-gnu

On Mac OS (with MacPorts) set:

    export PKG_LDFLAGS_OVERRIDE="-L/opt/local/lib -L/opt/local/lib/libgcc"

FastR necessarily provides very fine control over the versions of the llvm tools that are used, through environment variables all beginning with `FASTR_LLVM`. It also allows FastR to be completely independent from sulong. The following variables are mandatory:

`FASTR_LLVM_DRAGONEGG`: the location of the DragonEgg plugin (shared library built above)
`FASTR_LLVM_GFORTRAN`: a 4.6 version of `gfortran` (see `SULONG_GFORTRAN` above)
`FASTR_LLVM_GFORTRAN_LLVM_AS`: a 3.2 version of the `llvm-as` tool. N.B. this is created in sulong as part of the `DragonEgg` build. This can also be copied to a shared location.

The tools needed to build FastR and to install packages containing native code are `clang`, `clang++` and `opt`. Precisely which versions of these tools are used can be controlled in two ways.

1. Set `FASTR_LLVM_HOME` to a directory containing the tools, named as above. N.B. Since MacPorts uses names of the form `clang-mp-3.8` this requires using symbolic links. E.g. create a directory called, say, `llvm-3.8` and symlink to the MacPorts binary, e.g., `clang-mp-3.8` and name it as `clang`. Then set `FASTR_LLVM_HOME` to that directory.
2. Set `FASTR_LLVM_CLANG`, `FASTR_LLVM_CLANGPP` and `FASTR_LLVM_OPT` to the actual binary images, e.g., `export FASTR_LLVM_CLANG=/opt/local/bin/clang-mp-3.8`.

Not all versions of the llvm tools can compile FastR. `clang-3.2` that is downloaded as part of the DragonEgg build is known to work. This currently is placed in `sulong/cache/tools/llvm/bin` by the DragonEgg build and, again, can be copied to a shared location. `clang-3.8` and `clang-4.0` are known not to work, but `clang-3.9` does.

Once the above is set up, run `mx build`.

## Running

There is no compile-time dependency between FastR and Sulong; all communication is via the Truffle Interop API. Therefore Sulong must be dynamically imported using either `mx --dynamicimport sulong` or by setting the environment variable `DEFAULT_DYNAMIC_IMPORTS=sulong`, with latter being most convenient. With this in effect, a normal `mx R` will make Sulong available.

Note that if the `LLVM_PARSE_TIME` environment variable is set to any value, the time taken to parse each LLVM module is logged to the console, which is also a check that the LLVM implementation variant is being used.

# Implementation Details

## Compiler Wrapper Scripts

Compiler wrapper scripts are used to map the commands generated the `make` into an appropriate set of llvm tool invocations. This way no change is required to the `Makefile`s or the R package installation process.
Once Fastr is built, the scripts are stored in the `bin` sub-directory and are named: `llvm-cc`, `llvm-fc` and `llvm-c++`, for use during package installation.

The scripts normally operate silently. However, if `FASTR_LLVM_TOOLMODE`is set to `echorun`, the actual llvm tool commands resulting from the normal C/C++/Fortran command will be echoed to the standard output.

## Limitations
At the time of writing all the `RFFI` interfaces are implemented for LLVM. However, owing to a bug in DragonEgg, the actual Fortran code for the Lapack library is not executed under LLVM, only the wrapper.
